@@ -103,7 +103,7 @@ public :
     /** writes a given variant in the output vcf file
      */
     void writeVcfVariant(int bkt_id, string& chrom_name, uint64_t position, char* ref_char, char* alt_char, int repeat_size, string type);
-
+    void writeIndel(int bkt_id, string &chrom_name, uint64_t position, string ref_char, string alt_char, int repeat_size, string type);
 
     /*Getter*/
     /** Return the number of found breakpoints
@@ -130,7 +130,7 @@ public :
      */
     size_t kmer_size();
 
-    /** Return the numbre of max repeat 
+    /** Return the max repeat size at breakpoint
      */
     int max_repeat();
 
@@ -138,6 +138,9 @@ public :
      */
     int snp_min_val();
 
+    /** Return the threashold value of the branching filter
+    */
+    int branching_threshold();
     
     /** The last solid kmer before gap
      */
@@ -233,7 +236,18 @@ public :
     /** Incremente the value of backup_iterate
      */
     int backup_iterate();
-    
+
+    /*Incremente the value of homo_clean_indel
+     */
+    int homo_clean_indel_iterate();
+
+    /* Incremente the value of homo_fuzzy_indel
+     */
+    int homo_fuzzy_indel_iterate();
+    /* Incremente the value of hetero_indel
+     */
+    int hetero_indel_iterate();
+
     /*Setter*/
     /** Set value of recent_hetero
      */
@@ -465,8 +479,12 @@ void FindBreakpoints<span>::operator()()
                     v.push_back(token);
                 }
                 if(v[0]==m_chrom_name){ // we are on the current chromosome
-                    interval=std::make_pair(std::stoi(v[1]),std::stoi(v[2]));
-                    interval_vector.push_back( tuple<uint64_t ,uint64_t>(interval));
+                    uint64_t bed_begin = std::stoi(v[1]);
+                    uint64_t bed_end = std::stoi(v[2]);
+                    if ((bed_end-bed_begin) > this->finder->_kmerSize){
+                        interval=std::make_pair(std::stoi(v[1]),std::stoi(v[2]));
+                        interval_vector.push_back( tuple<uint64_t ,uint64_t>(interval));
+                    }
                 }
                 iss.clear();
             }
@@ -489,18 +507,28 @@ void FindBreakpoints<span>::operator()()
                         end_pos=get<1>(interval_vector.front());
                     }
                     
-                    if(!(*m_it_kmer).isValid() || (m_position<start_pos))
+                    if(!(*m_it_kmer).isValid())
                     {
-                        //Reintialize stretch_size for each bed region
-                        
+                        //Re-initialize stretch_size
                         this->m_solid_stretch_size = 0;
                         this->m_gap_stretch_size = 0;
                         this->m_kmer_begin = KmerCanonical();
                         this->m_kmer_end = KmerCanonical();
-                        //DEBUG
-                        //cout<<"n";
+
                     }
-                    
+
+                    if(m_position==start_pos-1) //for each beginning of bed region
+                    {
+                        //Re-initialize stretch_size for each bed region
+                        this->m_solid_stretch_size = 0;
+                        this->m_gap_stretch_size = 0;
+                        this->m_kmer_begin = KmerCanonical();
+                        this->m_kmer_end = KmerCanonical();
+                        
+                        //Re-initialize het_kmer_history for each bed region
+                        memset(this->m_het_kmer_history, 0, sizeof(info_type)*256);
+                    }
+
                     
                     if(((*m_it_kmer).isValid()) && (m_position>=start_pos)) //inside the current bed interval
                     {
@@ -648,7 +676,30 @@ void FindBreakpoints<span>::writeVcfVariant(int bkt_id, string& chrom_name, uint
 			repeat_size
 	);
 }
-
+template <size_t span>
+void FindBreakpoints<span>::writeIndel(int bkt_id, string &chrom_name, uint64_t position, string ref_string, string alt_string, int repeat_size, string type)
+{
+    // NOTE : currently all positions coming from FindObservers are 0-based, VCF is supposed to be 1-based, so we add +1
+    int variant_size = alt_string.length() - 1;
+    string GT = "./.";
+    if (type == "HOM")
+    {
+        GT = "1/1";
+    }
+    if (type == "HET")
+    {
+        GT = "0/1";
+    }
+    fprintf(this->finder->_vcf_file, "%s\t%lli\tbkpt%i\t%s\t%s\t.\tPASS\tTYPE=INS;LEN=%i;FUZZY=%i\tGT\t%s\n",
+            chrom_name.c_str(),
+            position + 1, //switch to 1-based
+            bkt_id,
+            ref_string.c_str(),
+            alt_string.c_str(),
+            variant_size,
+            repeat_size,
+            GT.c_str());
+}
 /*Getter*/
 template<size_t span>
 int FindBreakpoints<span>::node_in_branch(Node& kmer_node)
@@ -710,6 +761,12 @@ int FindBreakpoints<span>::snp_min_val()
     return this->finder->_snp_min_val;
 }
 
+template<size_t span>
+int FindBreakpoints<span>::branching_threshold()
+{
+    return this->finder->_branching_threshold;
+}
+
 /*Kmer related object*/
 template<size_t span>
 typename FindBreakpoints<span>::KmerCanonical& FindBreakpoints<span>::kmer_begin()
@@ -870,7 +927,22 @@ int FindBreakpoints<span>::backup_iterate()
 {
     return this->finder->_nb_backup++;
 }
+template <size_t span>
+int FindBreakpoints<span>::homo_clean_indel_iterate()
+{
+    return this->finder->_nb_homo_clean_indel++;
+}
 
+template <size_t span>
+int FindBreakpoints<span>::homo_fuzzy_indel_iterate()
+{
+    return this->finder->_nb_homo_fuzzy_indel++;
+}
+template <size_t span>
+int FindBreakpoints<span>::hetero_indel_iterate()
+{
+    return this->finder->_nb_hetero_indel++;
+}
 /*Setter*/
 template<size_t span>
 void FindBreakpoints<span>::recent_hetero(int value)
