@@ -27,6 +27,7 @@
 #include <FindInsertion.hpp>
 #include <FindSNP.hpp>
 #include <limits> //for std::numeric_limits
+#include <FindSmallInsertion.hpp>
 
 //#define PRINT_DEBUG
 /********************************************************************************/
@@ -63,6 +64,7 @@ Finder::Finder ()  : Tool ("MindTheGap find")
     _max_repeat = 0;
     _het_max_occ = 1;
     _snp_min_val = 5;
+    _branching_threshold = 5;
     _nbCores = 0;
     _breakpoint_file_name = "";
     _vcf_file_name = "";
@@ -75,14 +77,17 @@ Finder::Finder ()  : Tool ("MindTheGap find")
     _nb_solo_snp = 0;
     _nb_multi_snp = 0;
     _nb_backup = 0;
-    
+    _nb_homo_clean_indel = 0;
+    _nb_homo_fuzzy_indel = 0;
+    _nb_hetero_indel = 0;
     _homo_only = false;
     _homo_insert = true;
     _hete_insert = true;
     _snp = true;
     _backup = false;
     _deletion = true;
-    
+    _small_homo = true;
+
     _bed_file_name="";
 	
 	setHelp(&HelpFinder);
@@ -112,6 +117,7 @@ Finder::Finder ()  : Tool ("MindTheGap find")
     finderParser->push_front (new OptionNoParam (STR_INSERT_ONLY, "search only insertion breakpoints (do not report other variants)", false));
     //finderParser->getParser(STR_INSERT_ONLY)->setVisible(false);
     finderParser->push_front (new OptionOneParam (STR_HET_MAX_OCC, "maximal number of occurrences of a kmer in the reference genome allowed for heterozyguous breakpoints", false,"1"));
+    finderParser->push_front (new OptionOneParam (STR_BRANCHING_FILTER, "branching filter paramater for heterozygous insertions, maximal number of branching kmers in a 100-bp window before a heterozygous site (if -1 = no filter)", false,"15"));
     //allow to find heterozyguous breakpoints in n-repeated regions of the reference genome
     finderParser->push_front (new OptionOneParam (STR_MAX_REPEAT, "maximal repeat size detected for fuzzy sites", false, "5"));
     finderParser->push_front (new OptionNoParam (STR_HOMO_ONLY, "search only homozygous breakpoints", false));
@@ -305,7 +311,8 @@ void Finder::execute ()
     _max_repeat = getInput()->getInt(STR_MAX_REPEAT);
     _het_max_occ=getInput()->getInt(STR_HET_MAX_OCC);
     _snp_min_val=getInput()->getInt(STR_SNP_MIN_VAL);
-
+    _branching_threshold = getInput()->getInt(STR_BRANCHING_FILTER);
+    
     if(_het_max_occ<1){
     	_het_max_occ=1;
     }
@@ -318,6 +325,7 @@ void Finder::execute ()
 	_snp = true;
 	_backup = false;
 	_deletion = true;
+    _small_homo = true;
     }
     
     if(getInput()->get(STR_INSERT_ONLY) != 0)
@@ -328,6 +336,7 @@ void Finder::execute ()
 	_snp = false;
 	_backup = false;
 	_deletion = false;
+    _small_homo = true;
     }
 
     if(getInput()->get(STR_SNP_ONLY) != 0)
@@ -338,6 +347,7 @@ void Finder::execute ()
 	_snp = true;
 	_backup = false;
 	_deletion = false;
+    _small_homo = true;
     }
 
     if(getInput()->get(STR_DELETION_ONLY) != 0)
@@ -348,6 +358,7 @@ void Finder::execute ()
 	_snp = false;
 	_backup = false;
 	_deletion = true;
+    _small_homo = true;
     }
 
     if(getInput()->get(STR_HETERO_ONLY) != 0)
@@ -358,6 +369,7 @@ void Finder::execute ()
 	_snp = false;
 	_backup = false;
 	_deletion = false;
+    _small_homo = true;
     }
 
     if(getInput()->get(STR_WITH_BACKUP) != 0)
@@ -419,6 +431,10 @@ void Finder::resumeParameters(){
         getInfo()->add(2,"Graph",getInput()->getStr(STR_URI_GRAPH).c_str());
     }
     getInfo()->add(2,"Reference",getInput()->getStr(STR_URI_REF).c_str());
+    if(getInput()->get(STR_BED) != 0)
+    {
+        getInfo()->add(2,"Bed file",_bed_file_name.c_str());
+    }
     getInfo()->add(1,"Graph");
     getInfo()->add(2,"kmer-size","%i", _kmerSize);
 
@@ -456,6 +472,7 @@ void Finder::resumeParameters(){
     getInfo()->add(1,"Breakpoint detection options");
     getInfo()->add(2,"max_repeat","%i", _max_repeat);
     getInfo()->add(2,"hetero_max_occ","%i", _het_max_occ);
+    getInfo()->add(2,"branching filter value", "‰i", _branching_threshold);
     getInfo()->add(2,"homo_insertions","%s", _homo_insert ? "yes" : "no");
     getInfo()->add(2,"hete_insertions","%s", _hete_insert ? "yes" : "no");
     getInfo()->add(2,"snp","%s", _snp ? "yes" : "no");
@@ -474,6 +491,8 @@ void Finder::resumeResults(double seconds){
     getInfo()->add(3,"fuzzy","%i", _nb_hetero_fuzzy);
     getInfo()->add(1,"Other variants");
     getInfo()->add(2,"deletions","%i", _nb_clean_deletion+_nb_fuzzy_deletion);
+    getInfo()->add(2, "Homozygous insertions 1-2 bp size", "%i", _nb_homo_clean_indel + _nb_homo_fuzzy_indel);
+    getInfo()->add(2, "Heterozygous insertions 1-2 bp size", "%i", _nb_hetero_indel);
     //getInfo()->add(3,"clean", "%i", _nb_clean_deletion);
     //getInfo()->add(3,"fuzzy", "%i", _nb_fuzzy_deletion);
     getInfo()->add(2,"SNPs","%i", _nb_solo_snp+_nb_multi_snp);
@@ -540,8 +559,12 @@ void Finder::runFindBreakpoints<span>::operator ()  (Finder* object)
 	{
 		findBreakpoints.addGapObserver(new FindDeletion<span>(&findBreakpoints));
 	}
-	
-	if(object->_homo_insert)
+    if (object->_small_homo)
+    {
+        findBreakpoints.addGapObserver(new FindSmallCleanInsertion<span>(&findBreakpoints));
+        findBreakpoints.addGapObserver(new FindSmallFuzzyInsertion<span>(&findBreakpoints));
+    }
+    if(object->_homo_insert)
 	{
 		findBreakpoints.addGapObserver(new FindCleanInsertion<span>(&findBreakpoints));
 		findBreakpoints.addGapObserver(new FindFuzzyInsertion<span>(&findBreakpoints));
